% num = match(image1, image2)
%
% This function reads two images, finds their SIFT features, and
%   displays lines connecting the matched keypoints.  A match is accepted
%   only if its distance is less than distRatio times the distance to the
%   second closest match.
% It returns the number of matches displayed.
%
% Example: match('scene.pgm','book.pgm');

function [match] = lowe_match(des1,des2,distRatio)


if nargin==2
  distRatio = 0.45;  
end



    %im1 = rgb2gray(im1);
    %im2 = rgb2gray(im2);
    
    des1=double(des1');
    des2=double(des2');
    for i=1:size(des1,1)
        des1(i,:)=des1(i,:)/sqrt(sum(des1(i,:).^2));
    end
    for i=1:size(des2,1)
        des2(i,:)=des2(i,:)/sqrt(sum(des2(i,:).^2));
    end   

% For efficiency in Matlab, it is cheaper to compute dot products between
%  unit vectors rather than Euclidean distances.  Note that the ratio of 
%  angles (acos of dot products of unit vectors) is a close approximation
%  to the ratio of Euclidean distances for small angles.
%
% distRatio: Only keep matches in which the ratio of vector angles from the
%   nearest to second nearest neighbor is less than distRatio.
%distRatio = 0.45;   

% For each descriptor in the first image, select its match to second image.
des2t = des2';                          % Precompute matrix transpose
k=1;
match=[];
for i = 1 : size(des1,1)
   dotprods = des1(i,:) * des2t;        % Computes vector of dot products
   [vals,indx] = sort(acos(dotprods));  % Take inverse cosine and sort results

   % Check if nearest neighbor has angle less than distRatio times 2nd.
   if (vals(1) < distRatio * vals(2))
      %vals(1)/vals(2)
      match{k} = [i indx(1);];
      k=k+1;
      
   end
end


  